#ifndef __SXBOOT_H__
#define __SXBOOT_H__

#include "sxTypes.h"

#define SX_KERNEL_BASE				0x00100000
#define SX_KERNEL_SECTION_ALIGN		0x00001000
#define SX_KERNEL_TEXT_SIZE			0x00002000
#define SX_KERNEL_DATA_SIZE			0x00002000
#define SX_KERNEL_BSS_SIZE			0x00000000
#define SX_KERNEL_SIZE				(SX_KERNEL_TEXT_SIZE + SX_KERNEL_DATA_SIZE + SX_KERNEL_BSS_SIZE)

#define SX_KERNEL_STACK_SIZE		0x00001000
#define SX_KERNEL_STACK				SX_KERNEL_BASE + SX_KERNEL_SIZE + SX_KERNEL_STACK_SIZE

#define SX_BOOT_MAGIC				0x2badb002

#define SX_BOOTFLAG_MEMINFO			0
#define SX_BOOTFLAG_BOOTDEV			1
#define SX_BOOTFLAG_CMDLINE			2
#define SX_BOOTFLAG_MODULES			3
#define SX_BOOTFLAG_AOUT_HEADER		4
#define SX_BOOTFLAG_ELF_HEADER		5
#define SX_BOOTFLAG_MEMMAP			6
#define SX_BOOTFLAG_DRIVEINFO		7
#define SX_BOOTFLAG_CONFIGTABLE		8
#define SX_BOOTFLAG_LOADERNAME		9
#define SX_BOOTFLAG_APMINFO			10
#define SX_BOOTFLAG_GRAPHICS		11


typedef struct {
	sx_uint32 flags;
	sx_uint32 mem_lower;
	sx_uint32 mem_upper;
	sx_uint32 boot_device;
	sx_uint32 cmdline;
	sx_uint32 mods_count;
	sx_uint32 mods_addr;
	sx_uint32 _reserved0; /* these 4 fields contain information about */
	sx_uint32 _reserved1; /* a.out or ELF symbol tables, which are */
	sx_uint32 _reserved2; /* never set with our PE kernel. */
	sx_uint32 _reserved3; /* --- */
	sx_uint32 mmap_length;
	sx_uint32 mmap_addr;
	sx_uint32 drives_length;
	sx_uint32 drives_addr;
	sx_uint32 config_table;
	sx_uint32 loader_name;
	sx_uint32 apm_table;
	sx_uint32 vbe_control_info;
	sx_uint32 vbe_mode_info;
	sx_uint16 vbe_mode;
	sx_uint16 vbe_interface_seg;
	sx_uint16 vbe_interface_off;
	sx_uint16 vbe_interface_len;
} sx_multiboot_info;

typedef struct {
	sx_uint32 mod_start;
	sx_uint32 mod_end;
	sx_uint32 string;
	sx_uint32 reserved;
} sx_multiboot_module;

typedef struct {
	sx_uint32 size;
	sx_uint32 base_addr_low;
	sx_uint32 base_addr_high;
	sx_uint32 length_low;
	sx_uint32 length_high;
	sx_uint32 type;
} sx_multiboot_memory_map;

typedef struct {
	sx_uint32 size;
	sx_uint8 drive_number;
	sx_uint8 drive_mode;
	sx_uint16 drive_cylinders;
	sx_uint8 drive_heads;
	sx_uint8 drive_sectors;
	sx_uint16 drive_ports;
} sx_multiboot_drive_info;

typedef struct {
	sx_uint16 apm_version;
	sx_uint16 apm_cseg;
	sx_uint32 apm_offset;
	sx_uint16 apm_cseg16;
	sx_uint16 apm_dseg;
	sx_uint16 apm_flags;
	sx_uint16 apm_cseg_len;
	sx_uint16 apm_cseg16_len;
	sx_uint16 apm_dseg_len;
} sx_multiboot_apminfo;

#endif