#include "sxInterupts.h"
#include "sxIntHandlers.h"
#include "sxSegmentation.h"
#include "sxMain.h"
#include "sxConsole.h"
#include "sxDebug.h"

#define SX_ISR0(x)					\
	static SX_NAKED __sx_isr##x() {	\
		__asm cli					\
		__asm push 0				\
		__asm push x				\
		__asm jmp __sx_isr_common	\
	}

#define SX_ISR1(x)					\
	static SX_NAKED __sx_isr##x() {	\
		__asm cli					\
		__asm push x				\
		__asm jmp __sx_isr_common	\
	}

#define SX_IRQSR(x) SX_ISR0(x)
#define SX_IS_IRQ(x) (x >= SX_IRQ_BEGIN && x <= SX_IRQ_BEGIN + 15)

#define SX_PRINT_INTERUPT_INFO(i) {					\
		sx_eColors fg;								\
		SxConGetColor(&fg, SX_NULL);				\
		SxConSetTextColor(ecBrightRed);				\
		SxConPrint("\n !! Int: %d (code %d)\n", (i).interupt.number, (i).interupt.code);		\
		SxConPrint(" !!   * DS:  %p, EDI: %p, ESI: %p, EBP: %p,\n"	\
		" !!     ESP: %p, EBX: %p, EDX: %p, ECX: %p,\n"	\
		" !!     EAX: %p, Num: %p, Err: %p, EIP: %p,\n"	\
		" !!     CS:  %p, EFL: %b\n",	\
			(i).handler.ds, (i).handler.edi, (i).handler.esi, (i).handler.ebp, (i).handler.esp, (i).handler.ebx,	\
			(i).handler.edx, (i).handler.ecx, (i).handler.eax, (i).interupt.number, (i).interupt.code,	\
			(i).source.eip, (i).source.cs, (i).source.eflags);		\
		SxConSetTextColor(fg);						\
	}

static SX_ALIGN(8) sx_isr_t sxIntHandlers[SX_INTERUPT_COUNT] = {

	/* 0  Divide		*/	SxIntHandleDivide,
	/* 1  Debug			*/	SxIntUnhandledInt,
	/* 2  NMI			*/	SxIntUnhandledInt,
	/* 3  Breakpoint	*/	SxIntUnhandledInt,
	/* 4  Overflow		*/	SxIntUnhandledInt,
	/* 5  Bound Range	*/	SxIntUnhandledInt,
	/* 6  Invalid OP	*/	SxIntUnhandledInt,
	/* 7  Device N/A	*/	SxIntUnhandledInt,
	/* 8  Double Fault	*/	SxIntUnhandledInt,
	/* 9  CP Seg Overr.	*/	SxIntUnhandledInt,
	/* 10 Invalid TSS	*/	SxIntUnhandledInt,
	/* 11 Segment N/A	*/	SxIntUnhandledInt,
	/* 12 Stack Fault	*/	SxIntUnhandledInt,
	/* 13 General Prot.	*/	SxIntUnhandledInt,
	/* 14 Page Fault	*/	SxIntUnhandledInt,
	/* 15 Intel Reserv.	*/	SxIntUnhandledInt,
	/* 16 x87 FP Error	*/	SxIntUnhandledInt,
	/* 17 Align. Check	*/	SxIntUnhandledInt,
	/* 18 Machine Check	*/	SxIntUnhandledInt,
	/* 19 SIMD Except.	*/	SxIntUnhandledInt,
	/* 20 Intel Reserv.	*/	SX_NULL,
	/* 21 Intel Reserv.	*/	SX_NULL,
	/* 22 Intel Reserv.	*/	SX_NULL,
	/* 23 Intel Reserv.	*/	SX_NULL,
	/* 24 Intel Reserv.	*/	SX_NULL,
	/* 25 Intel Reserv.	*/	SX_NULL,
	/* 26 Intel Reserv.	*/	SX_NULL,
	/* 27 Intel Reserv.	*/	SX_NULL,
	/* 28 Intel Reserv.	*/	SX_NULL,
	/* 29 Intel Reserv.	*/	SX_NULL,
	/* 30 Intel Reserv.	*/	SX_NULL,
	/* 31 Intel Reserv.	*/	SX_NULL,

	/* 32 IRQ 0			*/	SxIntUnhandledIRQ,
	/* 33 IRQ 1			*/	SxIntUnhandledIRQ,
	/* 34 IRQ 2			*/	SxIntUnhandledIRQ,
	/* 35 IRQ 3			*/	SxIntUnhandledIRQ,
	/* 36 IRQ 4			*/	SxIntUnhandledIRQ,
	/* 37 IRQ 5			*/	SxIntUnhandledIRQ,
	/* 38 IRQ 6			*/	SxIntUnhandledIRQ,
	/* 39 IRQ 7			*/	SxIntUnhandledIRQ,
	/* 40 IRQ 8			*/	SxIntUnhandledIRQ,
	/* 41 IRQ 9			*/	SxIntUnhandledIRQ,
	/* 42 IRQ 10		*/	SxIntUnhandledIRQ,
	/* 43 IRQ 11		*/	SxIntUnhandledIRQ,
	/* 44 IRQ 12		*/	SxIntUnhandledIRQ,
	/* 45 IRQ 13		*/	SxIntUnhandledIRQ,
	/* 46 IRQ 14		*/	SxIntUnhandledIRQ,
	/* 47 IRQ 15		*/	SxIntUnhandledIRQ,

};

static __sx_isr_c(sx_interupt_info_t info) {
	if(info.interupt.number > SX_INTERUPT_COUNT || !sxIntHandlers[info.interupt.number]) {
		SxConPrint("No handler registered for Interupt %d. Further Information follows\n", info.interupt.number);
		SX_PRINT_INTERUPT_INFO(info);
		SxCoreHalt();
	}

	sxIntHandlers[info.interupt.number](info);

	if(SX_IS_IRQ(info.interupt.number)) {
		/* handle PICs... */
		if(info.interupt.number > (SX_IRQ_BEGIN + 8)) {
			/* slave PIC */
			_outp(SX_PIC_SLAVE_CMD, SX_PIC_EOI);
		}

		_outp(SX_PIC_MASTER_CMD, SX_PIC_EOI);
	}
}

static SX_NAKED __sx_isr_common() {
	__asm {
		pushad

		/* save data segment selector */
		xor eax, eax
		mov ax, ds
		push eax

		/* setup kernel data segment selector */
		mov ax, SX_KERNEL_DATA_SELECTOR
		mov ds, ax
		mov es, ax
		mov fs, ax
		mov gs, ax

		call __sx_isr_c

		/* restore the original data segment */
		pop eax
		mov ds, ax
		mov es, ax
		mov fs, ax
		mov gs, ax

		popad

		/* clean up interupt left overs, 4 bytes isr number, 4 bytes error code */
		add esp, 8
		sti
		iretd
	}
}

/* Common interupts (exceptions) */
SX_ISR0(0);   SX_ISR0(1);   SX_ISR0(2);   SX_ISR0(3);   SX_ISR0(4);
SX_ISR0(5);   SX_ISR0(6);   SX_ISR0(7);   SX_ISR0(8);   SX_ISR0(9);
SX_ISR1(10);  SX_ISR1(11);  SX_ISR1(12);  SX_ISR1(13);  SX_ISR1(14);
SX_ISR0(15);  SX_ISR0(16);  SX_ISR0(17);  SX_ISR0(18);  SX_ISR0(19);

/* Intel reserved */
SX_ISR0(20);  SX_ISR0(21);  SX_ISR0(22);  SX_ISR0(23);  SX_ISR0(24);
SX_ISR0(25);  SX_ISR0(26);  SX_ISR0(27);  SX_ISR0(28);  SX_ISR0(29);
SX_ISR0(30);  SX_ISR0(31);

/* Hardware Interupt Requests */
SX_IRQSR(32); SX_IRQSR(33); SX_IRQSR(34); SX_IRQSR(35); SX_IRQSR(36);
SX_IRQSR(37); SX_IRQSR(38); SX_IRQSR(39); SX_IRQSR(40); SX_IRQSR(41);
SX_IRQSR(42); SX_IRQSR(43); SX_IRQSR(44); SX_IRQSR(45); SX_IRQSR(46);
SX_IRQSR(47);

void SxIntHandlersInit() {
	/* setup standard exceptions and reserved entries */
	#define __SX_GATE(i) SxIntSetInteruptGate(i, (sx_uint32)&__sx_isr##i, SX_KERNEL_CODE_SELECTOR, 0)

	__SX_GATE(0);	__SX_GATE(1);	__SX_GATE(2);	__SX_GATE(3);	__SX_GATE(4);
	__SX_GATE(5);	__SX_GATE(6);	__SX_GATE(7);	__SX_GATE(8);	__SX_GATE(9);
	__SX_GATE(10);	__SX_GATE(11);	__SX_GATE(12);	__SX_GATE(13);	__SX_GATE(14);
	__SX_GATE(15);	__SX_GATE(16);	__SX_GATE(17);	__SX_GATE(18);	__SX_GATE(19);
	__SX_GATE(20);	__SX_GATE(21);	__SX_GATE(22);	__SX_GATE(23);	__SX_GATE(24);
	__SX_GATE(25);	__SX_GATE(26);	__SX_GATE(27);	__SX_GATE(28);	__SX_GATE(29);
	__SX_GATE(30);	__SX_GATE(31);

	/* Hardware IRQs */
	__SX_GATE(32);	__SX_GATE(33);	__SX_GATE(34);	__SX_GATE(35);	__SX_GATE(36);
	__SX_GATE(37);	__SX_GATE(38);	__SX_GATE(39);	__SX_GATE(40);	__SX_GATE(41);
	__SX_GATE(42);	__SX_GATE(43);	__SX_GATE(44);	__SX_GATE(45);	__SX_GATE(46);
	__SX_GATE(47);

	#undef __SX_GATE
}

void SxIntHandlerSet(sx_uint8 idx, sx_isr_t isr) {
	sxIntHandlers[idx] = isr;
}

void SxIntHandlerClear(sx_uint8 idx) {
	sxIntHandlers[idx] = SX_NULL;
}

void SxIrqHandlerSet(sx_uint8 idx, sx_isr_t isr) {
	sxIntHandlers[idx + 32] = isr;
}

void SxIrqHandlerClear(sx_uint8 idx) {
	sxIntHandlers[idx + 32] = SX_NULL;
}

/* Handlers */
void SxIntHandleDivide(sx_interupt_info_t info) {
	SxCorePanic("Divide by Zero Exception at %x:%p!", info.source.cs, info.source.eip);
}

void SxIntUnhandledInt(sx_interupt_info_t info) {
	SX_PRINT_INTERUPT_INFO(info);

	SxCorePanic("Unhandled exception %d at %x:%p!\n", info.interupt.number, info.source.cs, info.source.eip);
}

void SxIntUnhandledIRQ(sx_interupt_info_t info) {
	sx_eColors fg;

	SxConGetColor(&fg, SX_NULL);
	SxConSetTextColor(ecBrightBlue);

	SxConPrint("\n ... unhandled hardware interupt %d ... \n", info.interupt.number);

	SxConSetTextColor(fg);
}
