#include "sxSegmentation.h"
#include "sxConsole.h"
#include "sxTypes.h"

/* .type values for code and data descriptors */

#define SX_DESC_TYPE_CODE			(1 << 3)

#define SX_DESC_CODE_ACCESSED		(1)
#define SX_DESC_CODE_READABLE		(1 << 1)
#define SX_DESC_CODE_CONFORMING		(1 << 2)

#define SX_DESC_DATA_ACCESSES		(1)
#define SX_DESC_DATA_WRITEABLE		(1 << 1)
#define SX_DESC_DATA_EXPAND_DOWN	(1 << 2)

#pragma pack(push, 1)

typedef struct {
	sx_uint16 limit;
	sx_uint32 base;
} sx_gdtptr_t;

typedef struct {
	sx_uint16	limit_low;
	sx_uint16	base_low;
	sx_uint8	base_middle;
	sx_uint8	type		: 4;
	sx_uint8	system		: 1;
	sx_uint8	dpl			: 2;
	sx_uint8	present		: 1;
	sx_uint8	limit_high	: 4;
	sx_uint8	reserved	: 1;
	sx_uint8	long_code	: 1;
	sx_uint8	big			: 1;
	sx_uint8	granularity	: 1;
	sx_uint8	base_high;
} sx_gdt_segment_t;

typedef union {
	sx_gdt_segment_t	segment;
} sx_gdt_gate_t;

#pragma pack(pop)

/* I statically define the basic entries here, if others
 * are to be added, increase SX_GDT_ENTRIES (maybe we
 * need to switch to dynamic allocation sometimes?), and
 * add entries at runtime. */

static SX_ALIGN(8) sx_gdt_gate_t sxGdtEntries[] = {
/*	  limit		base	base	type					system	pl	p	limit	r	long	32bit	4KB		base */
	{ 0,		0,		0,		0,						0,		0,	0,	0,		0,	0,		0,		0,		0 },
	/* kernel mode (ring zero) code and data segments */
	{ 0xFFFF,	0,		0,		SX_DESC_TYPE_CODE,		1,		0,	1,	0xF,	0,	0,		1,		1,		0 },
	{ 0xFFFF,	0,		0,		SX_DESC_DATA_WRITEABLE,	1,		0,	1,	0xF,	0,	0,		1,		1,		0 },
	/* user mode (ring three) code and data segments */
	{ 0xFFFF,	0,		0,		SX_DESC_TYPE_CODE,		1,		3,	1,	0xF,	0,	0,		1,		1,		0 },
	{ 0xFFFF,	0,		0,		SX_DESC_DATA_WRITEABLE,	1,		3,	1,	0xF,	0,	0,		1,		1,		0 },
};

static SX_ALIGN(8) sx_gdtptr_t sxGdt = { (sizeof(sxGdtEntries) - 1), (sx_uint32)&sxGdtEntries };

void SxSegmentationInit() {
	__asm {
		lgdt	sxGdt

		mov ax, SX_KERNEL_DATA_SELECTOR
		mov ds, ax
		mov es, ax
		mov fs, ax
		mov gs, ax
		mov ss, ax
		
		; oh man, you don't know how long i've been trying until
		; i got the following to work.

		push eax
		mov eax, SX_KERNEL_CODE_SELECTOR
		push eax
		mov eax, [reload]
		push eax
		retf
	reload:
		pop eax
	}

	SxConPrint(" * Global Descriptor Table loaded from %p\n", &sxGdt);
}