#include "sxTimer.h"
#include "sxInterupts.h"
#include "sxIntHandlers.h"
#include "sxConsole.h"

#define SX_PIT_FREQUENCY	1193180
#define SX_PIT_PORT_CMD		0x43
#define SX_PIT_PORT_CH0		0x40
#define SX_PIT_PORT_CH1		0x41
#define SX_PIT_PORT_CH2		0x42

#define SX_PIT_MODE_IOTC	0	/* Interupt on Terminal Count */
#define SX_PIT_MODE_HWRTOS	1	/* Hardware re-triggerable one-shot */
#define SX_PIT_MODE_RATE	2	/* Rate generator */
#define SX_PIT_MODE_SQUARE	3	/* Square wave generator */
#define SX_PIT_MODE_STS		4	/* Software triggered strobe */
#define SX_PIT_MODE_HTS		5	/* Hardware triggered strobe */

#pragma pack(push, 1)

typedef union {
	struct {
		sx_uint8 bcd		: 1;
		sx_uint8 mode		: 3;
		sx_uint8 access		: 2;
		sx_uint8 channel	: 1;
	} cmd;

	sx_uint8 raw;
} sx_PitCommand;

#pragma pack(pop)

static void SxHandleTimerIRQ(sx_interupt_info_t info) {
	static int i = 0;
	SxConPrint("tick %d\n", i++);
}

void SxTimerInit(sx_uint32 interval) {
	sx_uint32 divisor = SX_PIT_FREQUENCY / interval;
	sx_PitCommand cmd;

	SxIrqHandlerSet(0, SxHandleTimerIRQ);

	cmd.cmd.bcd = 0;
	cmd.cmd.mode = SX_PIT_MODE_RATE;

	_outp(SX_PIT_PORT_CMD, cmd.raw);

	_outp(SX_PIT_PORT_CH0, divisor & 0xFF);
	_outp(SX_PIT_PORT_CH0, (divisor>>8) & 0xFF);

	SxConPrint(" * Timer divisor set to %d\n", divisor);

	SxUnmaskPICInterupt(0);
}

void SxTimerTest() {
	volatile sx_uint8 t;

	__asm {
	lbl:
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
		nop
	}

	_outp(SX_PIT_PORT_CMD, 0);
	t = _inp(SX_PIT_PORT_CH0);
	t |= (_inp(SX_PIT_PORT_CH0) << 8);

	//SxConPrint("%d\n", t);

	__asm {
		int 32
		jmp lbl
	}
}